import { world, system } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";

import * as MobsForm from './mobs.js';

const beastConfig = {
    "tree_ent": {
        name: "Tree Ent",
        texture: "textures/ui/luminous/tree_ent_photo",
        handler: MobsForm.infoTreeEnt
    },
    "rare_tree_ent": {
        name: "Cherry Tree Ent",
        texture: "textures/ui/luminous/rare_tree_ent_photo",
        handler: MobsForm.infoRareTreeEnt
    },
    "king_hermit": {
        name: "King Hermit",
        texture: "textures/ui/luminous/king_hermit_photo",
        handler: MobsForm.infoKingHermit
    },
    "rare_king_hermit": {
        name: "Golden King Hermit",
        texture: "textures/ui/luminous/rare_king_hermit_photo",
        handler: MobsForm.infoRareKingHermit
    },
    "mummy": {
        name: "Mummy",
        texture: "textures/ui/luminous/mummy_photo",
        handler: MobsForm.infoMummy
    },
    "rare_mummy": {
        name: "Red Mummy",
        texture: "textures/ui/luminous/rare_mummy_photo",
        handler: MobsForm.infoRareMummy
    },
    "yeti": {
        name: "Yeti",
        texture: "textures/ui/luminous/yeti_photo",
        handler: MobsForm.infoYeti
    },
    "rare_yeti": {
        name: "Arid Yeti",
        texture: "textures/ui/luminous/rare_yeti_photo",
        handler: MobsForm.infoRareYeti
    },
    "sea_viper": {
        name: "Sea Viper",
        texture: "textures/ui/luminous/sea_viper_photo",
        handler: MobsForm.infoSeaViper
    },
    "rare_sea_viper": {
        name: "Coral Sea Viper",
        texture: "textures/ui/luminous/rare_sea_viper_photo",
        handler: MobsForm.infoRareSeaViper
    },
    "toxic_gator": {
        name: "Toxic Gator",
        texture: "textures/ui/luminous/toxic_gator_photo",
        handler: MobsForm.infoToxicGator
    },
    "rare_toxic_gator": {
        name: "Frigid Toxic Gator",
        texture: "textures/ui/luminous/rare_toxic_gator_photo",
        handler: MobsForm.infoRareToxicGator
    }
};

export function show(player) {
    const actionForm = new ActionFormData()
        .title("Ultimate Bestiary")
        .body("This add-on adds the §6legendary beasts§r from Luminous, and a rarer, stronger variant of each to fight/hunt down, in order to get trophies, music disc, and new loot!");

    const keys = Object.keys(beastConfig);

    keys.forEach(key => {
        const beast = beastConfig[key];
        actionForm.button(beast.name, beast.texture);
    });

    actionForm.show(player).then(response => {
        if (response.canceled) {
            return;
        }

        const selectedKey = keys[response.selection];
        const beast = beastConfig[selectedKey];

        if (beast?.handler) {
            beast.handler(player);
        }
    });
}
