import { world, system, ItemStack } from "@minecraft/server"

import { Random } from 'utils/Random.js'
//import * as Random from 'utils/random.js'
import * as Vec3 from 'utils/vec3.js'
import * as UtilFunctions from 'utils/functions.js'

import 'items/ultimate_bestiary.js'


/* 
system.runInterval(() => {
    const players = world.getAllPlayers()
    players.forEach(player => {
        const { dimension, location } = player

        const biome = UtilFunctions.getBiome(dimension, location)
        world.sendMessage(biome.id)
    })
}, 20) */


system.runInterval(() => {
    const players = world.getAllPlayers()
    players.forEach(player => {
        const equippableComp = player.getComponent('equippable')
        const headItem = equippableComp.getEquipmentSlot('Head').getItem()
        const offhandItem = equippableComp.getEquipmentSlot('Offhand').getItem()


        if (headItem?.typeId == 'eu:shellmet') {
            const { dimension, location } = player
            const belowBlock = dimension.getBlockBelow(location, { maxDistance: 3 })
            if (belowBlock?.typeId.includes('sand')) {
                player.addEffect('speed', 20 * 2, { amplifier: 1, showParticles: false })
            }
        } else if (offhandItem?.typeId == 'eu:frosty_horn') {
            const { dimension, location } = player
            if (UtilFunctions.getBiome(dimension, location).id.includes('cold')) {
                player.addEffect('regeneration', 20 * 2, { showParticles: false })
            }
        } else if (offhandItem?.typeId == 'eu:sea_viper_tooth') {
            if (player.isInWater) {
                player.addEffect('resistance', 20 * 2, { amplifier: 1, showParticles: false })
                player.addEffect('conduict_power', 20 * 15, { amplifier: 0, showParticles: false })
            }
        }
    })
}, 20)


world.afterEvents.effectAdd.subscribe(ev => {
    const { effect, entity } = ev

    if (entity.typeId != 'minecraft:player') { return; }

    if (effect.typeId != 'poison' || effect.typeId != 'hunger') { return; }

    const equippableComp = entity.getComponent('equippable')
    const headItem = equippableComp.getEquipmentSlot('Head').getItem()

    if (headItem?.typeId == 'eu:mummy_wraps') {
        entity.removeEffect('poison')
        entity.removeEffect('hunger')
    }
})

world.afterEvents.entityHitEntity.subscribe(ev => {
    const { hitEntity } = ev
    if ((hitEntity.typeId == 'eu:king_hermit' || hitEntity.typeId == 'eu:rare_king_hermit') && hitEntity.getProperty('eu:is_hide')) {
        const { location, dimension } = hitEntity
        dimension.playSound('item.shield.block', location, { volume: 2, pitch: Random.int(8, 12) / 10 })
    }
})

world.afterEvents.projectileHitEntity.subscribe(ev => {
    const { dimension, location, projectile } = ev
    if (projectile.typeId == 'eu:thrown_yeti_breath') {
        const entity = ev.getEntityHit().entity
        dimension.spawnParticle('eu:yeti_explosion', location)
        entity.addEffect('slowness', 20 * 10, { amplifier: 7 })
    } else if (projectile.typeId == 'eu:thrown_rare_yeti_breath') {
        const entity = ev.getEntityHit().entity
        dimension.spawnParticle('eu:rare_yeti_explosion', location)
        entity.addEffect('poison', 20 * 60, { amplifier: 0 })
    }
})


world.afterEvents.entityHealthChanged.subscribe(ev => {
    const { entity, newValue } = ev

    if ((entity.typeId == 'eu:king_hermit' || entity.typeId == 'eu:rare_king_hermit') && !entity.hasTag('already_hide')) {
        const healthComp = entity.getComponent('health')
        if (newValue <= healthComp.effectiveMax / 2) {
            entity.addTag('already_hide')
            entity.triggerEvent('eu:start_hide')
        }

    } else if ((entity.typeId == 'eu:yeti' || entity.typeId == 'eu:rare_yeti') && entity.getProperty('eu:attack_states') == 'normal_attack' && !entity.hasTag('already_breath')) {
        const healthComp = entity.getComponent('health')
        if (newValue <= healthComp.effectiveMax / 3) {
            entity.addTag('already_breath')
            entity.triggerEvent('eu:start_ranged_attack')
        }
    }
})



world.afterEvents.entityHurt.subscribe(ev => {
    const { damageSource, hurtEntity } = ev
    const damagingEntity = damageSource.damagingEntity


    if (damagingEntity?.typeId == 'minecraft:player') {
        const equipableComp = damagingEntity.getComponent('equippable')
        const offhandItem = equipableComp.getEquipmentSlot('Offhand').getItem()

        if (offhandItem?.typeId == 'eu:tree_ent_root' && Random.int(1, 5) == 1) {
            hurtEntity.addEffect('slowness', 20 * 7)
        } else if (offhandItem?.typeId == 'eu:gator_tooth') {
            hurtEntity.addEffect('poison', 20 * 5)
        }
    } else if (damagingEntity?.typeId == 'eu:mummy' || damagingEntity?.typeId == 'eu:rare_mummy') {
        const variantComp = damagingEntity.getComponent('variant');
        const currentValue = variantComp.value;

        //0 -> green
        //1 -> pink
        //2 -> orange
        //3 -> blue

        if (currentValue == 0) {
            hurtEntity.addEffect('poison', 20 * 6)
        } else if (currentValue == 1) {
            const healthComponent = damagingEntity.getComponent('health')
            const currentHealth = healthComponent.currentValue
            healthComponent.setCurrentValue(currentHealth + 10)

        } else if (currentValue == 2) {
            hurtEntity.setOnFire(6)
        } else if (currentValue == 3) {
            hurtEntity.addEffect('blindness', 20 * 6)
            hurtEntity.addEffect('nausea', 20 * 6)

        }
    } else if ((damagingEntity?.typeId == 'eu:yeti' || damagingEntity?.typeId == 'eu:rare_yeti') && damagingEntity.getProperty('eu:attack_states') == 'normal_attack') {
        let attackCount = damagingEntity.getDynamicProperty('eu:count_attack');
        if (attackCount == undefined) {
            attackCount = 0;
        }
        attackCount += 1;
        damagingEntity.setDynamicProperty('eu:count_attack', attackCount);

        //world.sendMessage(attackCount.toString());

        if (attackCount >= 3) {
            damagingEntity.setDynamicProperty('eu:count_attack', 0);
            damagingEntity.triggerEvent('eu:start_strength_attack');
        }

    } else if (damagingEntity?.typeId == 'eu:sea_viper' || damagingEntity?.typeId == 'eu:rare_sea_viper') {
        const healthComp = damagingEntity.getComponent('health')
        if (Random.int(1, 3) == 1) {
            hurtEntity.playSound('mob.guardian.death', { volume: 2, pitch: Random.int(8, 12) / 10 })
            hurtEntity.addEffect('blindness', 20 * 5)
        }
        if (healthComp.currentValue <= healthComp.effectiveMax / 2) {
            let attackCount = damagingEntity.getDynamicProperty('eu:count_attack');
            if (attackCount == undefined) {
                attackCount = 0;
            }
            attackCount += 1;
            damagingEntity.setDynamicProperty('eu:count_attack', attackCount);
            //world.sendMessage(attackCount.toString());

            if (attackCount >= 5) {
                damagingEntity.setDynamicProperty('eu:count_attack', 0);
                hurtEntity.playSound('mob.elderguardian.curse', { volume: 1, pitch: Random.int(8, 12) / 10 })
                const view = damagingEntity.getViewDirection()
                hurtEntity.applyKnockback({ x: view.x * 4, z: view.z * 4 }, 1)

            }
        }
    } else if (damagingEntity?.typeId == 'eu:toxic_gator') {
        hurtEntity.addEffect('poison', 20 * 5)
        if (Random.int(1, 3) == 1) {
            hurtEntity.applyKnockback({ x: 0, z: 0 }, 1)
        }
    } else if (damagingEntity?.typeId == 'eu:rare_toxic_gator') {
        hurtEntity.addEffect('slowness', 20 * 5)
        if (Random.int(1, 3) == 1) {
            hurtEntity.applyKnockback({ x: 0, z: 0 }, 1)
        }
    }
    if ((hurtEntity.typeId == 'eu:mummy' || hurtEntity.typeId == 'eu:rare_mummy') && !hurtEntity.getProperty('eu:is_changing')) {
        const healthComp = hurtEntity.getComponent('health')

        if (!hurtEntity.hasTag('first_change') && healthComp.currentValue <= healthComp.effectiveMax * 2 / 3) {
            hurtEntity.addTag('first_change')
            hurtEntity.triggerEvent('eu:start_changing')
        }
        if (!hurtEntity.hasTag('second_change') && healthComp.currentValue <= healthComp.effectiveMax / 3) {
            hurtEntity.addTag('second_change')
            hurtEntity.triggerEvent('eu:start_changing')
        }

    } else if (hurtEntity?.typeId == 'eu:sea_viper' || hurtEntity?.typeId == 'eu:rare_sea_viper') {
        if (Random.int(1, 3) == 1) {
            damagingEntity?.playSound('mob.guardian.death', { volume: 2, pitch: Random.int(8, 12) / 10 })
            damagingEntity?.addEffect('mining_fatigue', 20 * 5)

        }
    }

})



function spawnHermitsAround(entity, count, range) {
    const location = entity.location;
    const dimension = entity.dimension;

    for (let i = 0; i < count; i++) {
        const offsetX = (Math.random() * 2 - 1) * range;
        const offsetZ = (Math.random() * 2 - 1) * range;
        const spawnLocation = {
            x: location.x + offsetX,
            y: location.y,
            z: location.z + offsetZ
        };
        dimension.spawnEntity('eu:sand_hermit', spawnLocation);

    }
}



world.afterEvents.dataDrivenEntityTrigger.subscribe(ev => {
    const { entity, eventId } = ev

    if ((entity.typeId == 'eu:tree_ent' || entity.typeId == 'eu:rare_tree_ent') && eventId == 'eu:destroy_nearby_leaves') {
        //world.sendMessage('destruyendo hojas')
        const { location, dimension } = entity
        for (let i = -2; i <= 2; i++) {
            for (let j = -2; j <= 1; j++) {
                for (let k = -2; k <= 2; k++) {
                    const block = dimension.getBlock({ x: location.x + i, y: location.y + j + 2, z: location.z + k })
                    if (block?.typeId.includes('leaves')) {
                        dimension.runCommand(`setblock ${block.location.x} ${block.location.y} ${block.location.z} air destroy`)
                    }
                }
            }
        }
    } else if (entity.typeId == 'eu:king_hermit' || entity.typeId == 'eu:rare_king_hermit') {
        if (eventId == 'eu:start_hide') {
            //world.sendMessage('entrando al modo escondido')
            spawnHermitsAround(entity, 3, 2)
            entity.addEffect('regeneration', 20 * 10, { amplifier: 3 })
        } else if (eventId == 'eu:end_hide') {
            //world.sendMessage('saliendo del modo escondido')
            entity.playAnimation('animation.king_hermit.unhide')
        }
    } else if (entity.typeId == 'eu:mummy' || entity.typeId == 'eu:rare_mummy') {

        if (eventId == 'eu:start_changing') {
            entity.addEffect('resistance', 20 * 2.5, { amplifier: 1, showParticles: false })
        } else if (eventId == 'eu:end_changing') {
            //world.sendMessage('ya me transforme')

            const { location, dimension } = entity
            dimension.playSound('block.enchanting_table.use', location, { volume: 2, pitch: Random.int(8, 12) / 10 })
            const healthComponent = entity.getComponent('health')
            const currentHealth = healthComponent.currentValue
            healthComponent.setCurrentValue(currentHealth + 20)

            const variants = [0, 1, 2, 3];
            const variantComp = entity.getComponent('variant');
            const currentValue = variantComp.value;
            const possibleVariants = variants.filter(v => v != currentValue);
            const newVariant = Random.pick(possibleVariants);

            switch (newVariant) {
                case 0: {
                    dimension.spawnParticle('eu:poison_mini_explosion', location)
                    entity.triggerEvent('eu:set_green_variant')
                    break;
                }
                case 1: {
                    dimension.spawnParticle('eu:normal_hearth_particle', location)
                    entity.triggerEvent('eu:set_pink_variant')
                    break;
                }
                case 2: {
                    dimension.spawnParticle('eu:flame', location)
                    entity.triggerEvent('eu:set_orange_variant')
                    break;
                }
                case 3: {
                    dimension.spawnParticle('eu:soul_flame', location)
                    entity.triggerEvent('eu:set_blue_variant')
                    break;
                }
                default: break;
            }
        }
    } else if (entity.typeId == 'eu:yeti' || entity.typeId == 'eu:rare_yeti') {
        if (eventId == 'eu:start_strength_attack') {
            const { dimension, location } = entity

            system.runTimeout(() => {
                if (entity.isValid) {
                    const explosionLocation = Vec3.getLocalCoordinates(entity, { x: 3, y: 0, z: 0 })
                    //world.sendMessage('pum ataque')
                    dimension.spawnParticle('eu:yeti_explosion', explosionLocation)
                    dimension.createExplosion(explosionLocation, 3, { breaksBlocks: false, source: entity })
                }
            }, 20)
        } else if (eventId == 'eu:start_ranged_attack') {
            const { dimension, location } = entity

            //Modera la fuerza o el proyectil ira tan rapido que ni se vera XD
            for (let i = 1; i <= 3; i++) {
                system.runTimeout(() => {
                    if (entity.isValid) {
                        //world.sendMessage('bola numero: ' + i.toString())
                        const target = entity.target
                        //world.sendMessage(`${target.nameTag} x:${target.location.x} y:${target.location.y} z:${target.location.z}`)
                        const localLocation = Vec3.getLocalCoordinates(entity, { x: 1.5, y: 2.5, z: 0 })
                        //ojo importante usar la ubicacion local nueva para calcular la direccion
                        const direction = Vec3.subtract(target.getHeadLocation(), localLocation)

                        entity.lookAt(target.location)
                        const projectileType = entity.typeId == 'eu:yeti' ? 'eu:thrown_yeti_breath' : 'eu:thrown_rare_yeti_breath';

                        UtilFunctions.shootProjectile(projectileType, dimension, localLocation, direction, 0.2)
                    }
                }, 15 * i)
            }
        }
    }
})



system.beforeEvents.startup.subscribe(ev => {
    const { blockComponentRegistry } = ev

    blockComponentRegistry.registerCustomComponent('eu:sea_viper_egg', ({
        onPlayerDestroy: arg => {
            const { block, dimension, destroyedBlockPermutation, player } = arg
            const equippableComp = player.getComponent('equippable')
            const item = equippableComp.getEquipment('Mainhand')
            const enchantableComp = item?.getComponent('enchantable')
            if (!enchantableComp?.hasEnchantment('silk_touch')) {
                const entityMap = {
                    'eu:sea_viper_egg': 'eu:sea_viper',
                    'eu:rare_sea_viper_egg': 'eu:rare_sea_viper'
                };
                const entityId = entityMap[destroyedBlockPermutation.type.id];
                dimension.spawnEntity(entityId, block.location)
            }

        }
    }))
})

world.afterEvents.entityDie.subscribe(ev => {
    const { deadEntity } = ev
    if (deadEntity.typeId == 'eu:phoenix') {
        const { location, dimension } = deadEntity
        const rotation = deadEntity.getRotation()
        deadEntity.remove()
        dimension.spawnParticle('eu:rebirth_phoenix', { x: location.x, y: location.y + 1.5, z: location.z })
        const entity = dimension.spawnEntity('eu:baby_phoenix', location)
        entity.setRotation(rotation)

    }
})

/* 
system.runInterval(() => {
    const players = world.getAllPlayers()
    players.forEach(player => {
        const rotation = player.getRotation()
        player.onScreenDisplay.setActionBar(`x:${rotation.x} y:${rotation.y}`)
    })
})
 */


/* 
world.afterEvents.itemUse.subscribe(ev => {
    const { source } = ev;
    const { dimension, location } = source;

    const armorStand = dimension.getEntities({ closest: 1, location: location, type: 'minecraft:armor_stand' })[0]
    const rotation = armorStand.getRotation()
    world.sendMessage(`x: ${rotation.x} y: ${rotation.y}`)

    armorStand.lookAt(source.location)

});
 */
