export const Random = {
    // Numeros aleatorios
    random: () => Math.random(),

    number(min, max) {
        return Math.random() * (max - min) + min;
    },

    int(min, max, step = 1) {
        const range = Math.floor((max - min) / step);
        return min + step * Math.floor(Math.random() * (range + 1));
    },

    bigint(min, max) {
        min = BigInt(min);
        max = BigInt(max);
        return min + BigInt(Math.floor(Math.random() * Number(max - min + 1n)));
    },

    boolean(prob = 0.5) {
        return Math.random() < prob;
    },

    // Seleccion aleatoria
    shuffle(arr) {
        for (let i = arr.length - 1; i > 0; i--) {
            const j = Math.floor(Math.random() * (i + 1));
            [arr[i], arr[j]] = [arr[j], arr[i]];
        }
        return arr;
    },

    toShuffled(arr) {
        return Random.shuffle([...arr]);
    },

    pick(arr) {
        if (!arr.length) return undefined;
        return arr[Math.floor(Math.random() * arr.length)];
    },

    take(arr, count) {
        const copy = Random.toShuffled(arr);
        return copy.slice(0, count);
    },

    sample(arr, options = {}) {
        const weights = options.weights || arr.map(() => 1);
        const total = weights.reduce((a, b) => a + b, 0);
        let r = Math.random() * total;
        for (let i = 0; i < arr.length; i++) {
            if (r < weights[i]) return arr[i];
            r -= weights[i];
        }
        return arr[arr.length - 1];
    },

    // Bytes aleatorios
    /*     
        bytes(count) {
            const buf = new Uint8Array(count);
            crypto.getRandomValues(buf);
            return buf;
        },
    
        fillBytes(buf) {
            crypto.getRandomValues(buf);
            return buf;
        } */
};

/* Ejemplo de uso
console.log(Random.random());               // Entre 0 y 1
console.log(Random.number(0, 10, 2));       // 0, 2, 4, 6, 8, 10
console.log(Random.boolean(0.1));           // true con 10% probabilidad
console.log(Random.shuffle([1, 2, 3, 4]));  // Mutacion
console.log(Random.toShuffled([1, 2, 3]));  // Copia desordenada
console.log(Random.take(["a", "b", "c"], 2));
console.log(Random.sample(["cat", "dog", "fox"], { weights: [1, 1, 10] }));
console.log(Random.bytes(5));
console.log(Random.pick(['manzana', 'banana', 'cereza', 'durazno'])); // Devuelve una fruta al azar
 */