

import { BiomeTypes, BiomeType } from "@minecraft/server";
BiomeType.toString()
/**
 * @param {Dimension} dimension
 * @param {Vector3} location
 * @param {Boolean} maxAccuracy
 * @returns {BiomeType}
 */
export function getBiome(dimension, location, maxAccuracy = false) {
    const locations = []

    for (let type of BiomeTypes.getAll()) {
        const loc = dimension.findClosestBiome(location, type.id, { boundingSize: { x: 64, y: 64, z: 64 } })
        if (loc != undefined) {
            if (maxAccuracy) locations.push({ location: loc, type: type });
            else return type
        }
    }

    if (maxAccuracy) {
        const getDist = ((vec1, vec2) => Math.sqrt((vec1.x - vec2.x) ** 2 + (vec1.y - vec2.y) ** 2 + (vec1.z - vec2.z) ** 2));
        locations.sort((biome1, biome2) => getDist(biome1.location, location) - getDist(biome2.location, location))
        return locations[0]?.type
    }
}

export function getCardinalDirection(entity) {
    const yaw = entity.getRotation().y;
    if (yaw >= -45 && yaw < 45) return 'north';
    else if (yaw >= 45 && yaw < 135) return 'east';
    else if (yaw >= 135 || yaw < -135) return 'south';
    else return 'west';
}

export function getRandomElement(array) {
    const indiceAleatorio = Math.floor(Math.random() * array.length);
    return array[indiceAleatorio];
}

export function hasEquipmentSlot(player, itemID, slot) {
    let getEquipment = player.getComponent('equippable').getEquipment(slot)
    if (getEquipment?.typeId == itemID) {
        return true;
    }
}

export function hasItem(player, itemId) {
    let inventory = player.getComponent("inventory");
    if (!inventory) {
        return false;
    }
    let container = inventory.container;
    for (let i = 0; i < container.size; i++) {
        let item = container.getItem(i);
        if (item?.typeId == itemId) {
            return true;
        }
    }
    return false;
}


export function updateItemDurability(source, item, durabilityModifier) {
    const equippable = source.getComponent("equippable");
    const durability = item.getComponent("durability");

    durability.damage += durabilityModifier;

    const maxDurability = durability.maxDurability
    const currentDamage = durability.damage
    if (currentDamage >= maxDurability) {
        source.playSound('random.break', { pitch: 1, location: source.location, volume: 1 })
        equippable.setEquipment("Mainhand", undefined);
    }
    else if (currentDamage < maxDurability) {

        equippable.setEquipment("Mainhand", item);
    }
}

export function randomInt(min, max) {

    return Math.floor(Math.random() * (max - min + 1)) + min;
}


/**
 * 
 * @param {String} projectileId 
 * @param {Dimension} dimension 
 * @param {Location} location 
 * @param {Vector3} direction 
 * @param {Number} velocityMultiplier 
 * @param {Number} uncertainty 
 */
export function shootProjectile(projectileId, dimension, location, direction, velocityMultiplier = 0.5, uncertainty = 0) {
    let velocity = { x: direction.x * velocityMultiplier, y: direction.y * velocityMultiplier, z: direction.z * velocityMultiplier };
    const projectile = dimension.spawnEntity(projectileId, location);
    const projectileComp = projectile.getComponent('minecraft:projectile');

    projectileComp?.shoot(velocity, {
        uncertainty: uncertainty,
    });
}
